@@ -187,9 +187,10 @@ void zbat_getinfo(struct zbat *zbat){
 #define GBS_ONBATTERY  0x2
 
 void zbat_getinfo(struct zbat *zbat){
-    ULONG sum =0UL, n = 0UL;
+    ULONG sum =0UL, n = 1UL;
 #if 0
 	int idev;
+    n = 0UL;
 
 
     HDEVINFO hdev = SetupDiGetClassDevs(&GUID_DEVCLASS_BATTERY, 0, 0, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);
@@ -255,7 +256,10 @@ void zbat_getinfo(struct zbat *zbat){
     SetupDiDestroyDeviceInfoList(hdev);
 #endif   
     zbat->n = n;
-    if (n > 0) zbat->capacity = sum / n;
+    if (n > 0) 
+        zbat->capacity = sum / n;
+    else
+        zbat->capacity = 0;
 }
 #endif
 
