@@ -158,7 +158,7 @@ static void zhttp_prepare1(struct zhttp *http, struct zselect *zsel, const char
 		}
 		http->port = 443;
 #else
-		zhttp_disconnect(http, g_strdup(g_strdup("TLS not supported")));
+		zhttp_disconnect(http, g_strdup("TLS not supported"));
 #endif
 	}else{
 		c = z_strcasestr(u, "http:");
@@ -292,7 +292,7 @@ void zhttp_get(struct zhttp *http, struct zselect *zsel, const char *url, void (
 
 void zhttp_add_header(struct zhttp *http, const char *name, const char *value)
 {
-	g_hash_table_insert(http->headers, (char *)name, (char *)value);
+	g_hash_table_insert(http->headers, g_strdup(name), g_strdup(value));
 }
 
 void zhttp_raw(struct zhttp *http, struct zselect *zsel, const char *url, const char *raw_request,  void (*callback)(struct zhttp *), void *arg){
@@ -396,15 +396,17 @@ static void zhttp_adns_callback(struct zasyncdns *adns, int n, int *family, int
 	int port = http->port;
 
     if (errorstr != NULL){
-        g_free(http->errorstr);
-        http->errorstr = g_strdup_printf("Can't resolve %s: %s", http->server, errorstr);
+		http->state = ZHTTPST_ERROR;
+		g_free(http->errorstr);
+		http->errorstr = g_strdup_printf("Can't resolve %s: %s", http->server, errorstr);
         http->callback(http);
         return;
     }
 
     if (n == 0){
-        g_free(http->errorstr);
-        http->errorstr = g_strdup_printf("Can't resolve %s: No data returned", http->server);
+		http->state = ZHTTPST_ERROR;
+		g_free(http->errorstr);
+		http->errorstr = g_strdup_printf("Can't resolve %s: No data returned", http->server);
         http->callback(http);
         return;
     }
@@ -464,8 +466,9 @@ static void zhttp_adns_callback(struct zasyncdns *adns, int n, int *family, int
 				http->state = ZHTTPST_CONNECTING;
 				http->connecting_timer_id = zselect_timer_new(http->zsel, 30000, zhttp_connecting_timer, http);
             }else{                
-                g_free(http->errorstr);
-				http->errorstr = g_strdup_printf("Can't connect to %s, %s", http->serveraddr, z_sock_strerror());                
+				http->state = ZHTTPST_ERROR;
+				g_free(http->errorstr);
+				http->errorstr = g_strdup_printf("Can't connect to %s, %s", http->serveraddr, z_sock_strerror());
                 closesocket(http->sock);                
                 http->sock = -1;         
             }           
@@ -738,6 +741,7 @@ int zhttp_write_data(struct zhttp *http, const char *filename){
 	towrite = http->response->len - http->dataofs;
 	
 	if (towrite <= 0 || http->dataofs <= 0){
+		http->state = ZHTTPST_ERROR;
 		g_free(http->errorstr);
 		http->errorstr = g_strdup_printf("Bad http response (len=%d ofs=%d)", http->response->len, http->dataofs);
 		return -1;
@@ -746,6 +750,7 @@ int zhttp_write_data(struct zhttp *http, const char *filename){
 	z_wokna(wfn);
 	f = fopen(wfn, "wb");
 	if (!f){
+		http->state = ZHTTPST_ERROR;
 		g_free(http->errorstr);
 		http->errorstr = g_strdup_printf("Can't write %s", wfn);
 		g_free(wfn);
@@ -753,6 +758,8 @@ int zhttp_write_data(struct zhttp *http, const char *filename){
 	}
 	ret = fwrite(http->response->buf + http->dataofs, 1, towrite, f);
 	if (ret != towrite){
+        fclose(f);
+		http->state = ZHTTPST_ERROR;
 		g_free(http->errorstr);
 		http->errorstr = g_strdup_printf("Can't write to %s", wfn);
 		g_free(wfn);
@@ -858,14 +865,15 @@ void zhttp_init_tls(void){
 char *http_get_header(struct zhttp *http, const char *header_name){
 
 	if (http->dataofs < 0) return NULL;
+	if (http->response == NULL) return NULL;
+	if (http->response->buf == NULL) return NULL;
 
 	char *c;
 	for (c = http->response->buf; *c != '\0'; c++){
 		char *colon = strchr(c, ':');
-		char *lf = strchr(colon + 1, '\n');
-		int n = Z_MIN(colon - c, lf - c);
-
-		if (colon != NULL){
+		char *lf = strchr(c, '\n');
+		if (colon != NULL && lf != NULL){
+			int n = Z_MIN(colon - c, lf - c);
 			if (strncasecmp(c, header_name, n) == 0){
 				char *ct = g_strndup(colon + 1, lf - (colon + 1));
 				char *ret = z_strdup_trim(ct);
@@ -873,6 +881,7 @@ char *http_get_header(struct zhttp *http, const char *header_name){
 				return ret;
 			}
 		}
+		if (lf == NULL) break;
 		c = lf;
 	}
 	return NULL;
@@ -884,7 +893,22 @@ int http_is_content_type(struct zhttp *http, const char *content_type){
 	char *ct = http_get_header(http, "content-type");
 	if (!ct) return ret;
 
+	z_strip_from(ct, ';');
+
 	ret = (strcasecmp(ct, content_type) == 0);
 	g_free(ct);
 	return ret;
 }
+
+void zhttp_auth_basic(struct zhttp* http, const char* username, const char* password) {
+	if (!username || !*username || !password) return;
+
+	GString* gs = g_string_sized_new(100);
+	g_string_append(gs, "Basic ");
+	char* a = g_strdup_printf("%s:%s", username, password);
+	zg_string_eprintfa("b", gs, "%b", a, strlen(a));
+	zhttp_add_header(http, "Authorization", gs->str);
+
+	g_free(a);
+	g_string_free(gs, TRUE);
+}
\ No newline at end of file
