@@ -1,6 +1,6 @@
 /*
     zsdl.c - reusable SDL functions
-    Copyright (C) 2011-2020 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2011-2023 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -29,6 +29,7 @@
 #include <sys/wait.h>
 #endif
 
+#include <errno.h>
 #include <math.h>
 
 #include <zdebug.h>
@@ -112,13 +113,11 @@ struct zzsdl *zsdl_init(SDL_Surface *screen, int font_h, int inverse){
 	outline9x16 = zfont_create_outline(font9x16, sizeof(font9x16), 16);
 
     //zsdl_font_save(screen);
-    //zsdl_font_dump("e:\\down\\tmp\\font13x24.png", "font13x24.inc");
-//    zsdl_font_dump("e:\\down\\tmp\\font9x16.png", "font9x16.inc");
-//    zsdl_font_dump("font9x16.png", "font9x16.inc");
-//    zsdl_font_dump("font13x24.png", "font13x24.inc");
-//    zsdl_font_dump_skip_red("font8x13red.png", "font8x13.inc");
-//    zsdl_font_dump("font8x13.png", "font8x13.inc");
-    //zsdl_font_dump("font18x32apng", "font18x32.inc");
+	//zsdl_font_dump("font8x13.png", "font8x13.inc");
+	//zsdl_font_dump("font9x16.png", "font9x16.inc");
+ //   zsdl_font_dump("font13x24.png", "font13x24.inc");
+	//zsdl_font_dump("font18x32.png", "font18x32.inc");
+	//    zsdl_font_dump_skip_red("font8x13red.png", "font8x13.inc");
    return zsdl;
 }
 
@@ -319,7 +318,7 @@ void z_putpixel24(SDL_Surface *surface, int x, int y, int color){
     Uint8  *p8;
 
     if (z_clip_pixel(&surface->clip_rect, x, y)) return;
-    p16 = (Uint16 *) surface->pixels + y*surface->pitch/3 + x;
+    p16 = (Uint16 *) ((Uint8 *)surface->pixels + y*surface->pitch + x*3);
     p8  = (Uint8 *)  (p16+1); 
     *p16 = color;     /* TODO Big endian? */
     *p8  = color>>16; 
@@ -354,7 +353,7 @@ void z_putpixel24inv(SDL_Surface *surface, int x, int y, int color){
     Uint8  *p8;
 
     if (z_clip_pixel(&surface->clip_rect, x, y)) return;
-    p16 = (Uint16 *) surface->pixels + y*surface->pitch/3 + x;
+    p16 = (Uint16 *) ((Uint8 *)surface->pixels + y*surface->pitch + x*3);
     p8  = (Uint8 *)  (p16+1); 
     *p16 = ~color;     /* TODO Big endian? */
     *p8  = ~(color>>16); 
@@ -682,7 +681,7 @@ void zsdl_printf(SDL_Surface *surface, int x, int y, int color, int bgcolor, int
 
 void zsdl_fit(int *flg, int width, int height, char *str){
 	int widths[] = { 32, 24, 16, 13, 0 };
-	int i, fw, fh;
+	int i, fw, fh = 16;
 	
 	for (i = 0; widths[i] != 0; i++){
 		fh = widths[i];
@@ -866,6 +865,8 @@ void z_do_line(SDL_Surface *surface, int x1, int y1, int x2, int y2, int color,
     int dx, dy, p;  
     int inc, tmp;
     
+    func(NULL, 0, 0, 0); // reset state (dashcnt)
+
     dx=abs(x1-x2);
     dy=abs(y1-y2);
     if (dx>=dy){
@@ -1063,12 +1064,26 @@ void z_circle(SDL_Surface *surface, int x, int y, int r, int c){
 }
 
 
-int dashcnt = 0;
+static int dashcnt = 0;
 void z_dashfce(SDL_Surface *surface, int x, int y, int d) {
+    if (surface == NULL) {
+        dashcnt = 0;
+        return;
+    }
     if (dashcnt++%6<3) return;
     z_putpixel(surface, x, y, d);
 }
-    
+
+static int dotcnt = 0;
+void z_dotfce(SDL_Surface* surface, int x, int y, int d) {
+    if (surface == NULL) {
+        dotcnt = 0;
+        return;
+    }
+    if (dotcnt++ % 2 == 0) return;
+    z_putpixel(surface, x, y, d);
+}
+
 
 void z_rect(SDL_Surface *surface, int x1, int y1, int x2, int y2, int c){
     z_line(surface, x1, y1, x2, y1, c);
@@ -1487,9 +1502,10 @@ void z_disable_screensaver(void){
     //    system("xset -dpms ; xset s off");
         pid_t pid=fork();
         if (pid==0){
-            execlp("xset", "xset", "s", "off", "-dpms", NULL);
-            exit(-1);
-        }else{
+            int ret = execlp("xset", "xset", "s", "off", "-dpms", NULL);
+            error("execlp xset failed, ret=%d errno=%d\n", ret, errno);
+			exit(-1);
+		}else{
             dbg("fork() for xset %d\n", pid);
         }
         waitpid(pid, NULL, 0);
@@ -1564,7 +1580,7 @@ int zsdl_maximized(struct zzsdl *zsdl, int *w, int *h){
 }
 
 int z_color_from_html(char *htmlcolor){
-	int r, g, b, n;
+	unsigned int r, g, b, n;
 	char *c = htmlcolor;
 	if (*c == '#') c++;
 
